@@ -1,12 +1,26 @@
-/*
-    SPDX-FileCopyrightText: 2015 Martin Gräßlin <mgraesslin@kde.org>
+/****************************************************************************
+Copyright 2015  Martin Gräßlin <mgraesslin@kde.org>
 
-    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
-*/
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) version 3, or any
+later version accepted by the membership of KDE e.V. (or its
+successor approved by the membership of KDE e.V.), which shall
+act as a proxy defined in Section 6 of version 3 of the license.
+
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+****************************************************************************/
 #include "server_decoration_interface.h"
 #include "display.h"
 #include "global_p.h"
-#include "logging.h"
+#include "logging_p.h"
 #include "resource_p.h"
 #include "surface_interface.h"
 
@@ -18,6 +32,7 @@ namespace KWayland
 {
 namespace Server
 {
+
 class ServerSideDecorationManagerInterface::Private : public Global::Private
 {
 public:
@@ -25,19 +40,18 @@ public:
 
     Mode defaultMode = Mode::None;
 
-    QVector<wl_resource *> resources;
+    QVector<wl_resource*> resources;
 
 private:
     void bind(wl_client *client, uint32_t version, uint32_t id) override;
 
     static void unbind(wl_resource *resource);
-    static Private *cast(wl_resource *r)
-    {
-        return reinterpret_cast<Private *>(wl_resource_get_user_data(r));
+    static Private *cast(wl_resource *r) {
+        return reinterpret_cast<Private*>(wl_resource_get_user_data(r));
     }
 
-    static void createCallback(wl_client *client, wl_resource *resource, uint32_t id, wl_resource *surface);
-    void create(wl_client *client, wl_resource *resource, uint32_t id, wl_resource *surface);
+    static void createCallback(wl_client *client, wl_resource *resource, uint32_t id, wl_resource * surface);
+    void create(wl_client *client, wl_resource *resource, uint32_t id, wl_resource * surface);
 
     ServerSideDecorationManagerInterface *q;
     static const struct org_kde_kwin_server_decoration_manager_interface s_interface;
@@ -46,13 +60,15 @@ private:
 
 const quint32 ServerSideDecorationManagerInterface::Private::s_version = 1;
 
-#ifndef K_DOXYGEN
-const struct org_kde_kwin_server_decoration_manager_interface ServerSideDecorationManagerInterface::Private::s_interface = {createCallback};
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+const struct org_kde_kwin_server_decoration_manager_interface ServerSideDecorationManagerInterface::Private::s_interface = {
+    createCallback
+};
 #endif
 
 void ServerSideDecorationManagerInterface::Private::createCallback(wl_client *client, wl_resource *resource, uint32_t id, wl_resource *surface)
 {
-    reinterpret_cast<Private *>(wl_resource_get_user_data(resource))->create(client, resource, id, surface);
+    reinterpret_cast<Private*>(wl_resource_get_user_data(resource))->create(client, resource, id, surface);
 }
 
 void ServerSideDecorationManagerInterface::Private::create(wl_client *client, wl_resource *resource, uint32_t id, wl_resource *surface)
@@ -70,8 +86,16 @@ void ServerSideDecorationManagerInterface::Private::create(wl_client *client, wl
         delete decoration;
         return;
     }
+    QObject::connect(s, &QObject::destroyed, decoration,
+        [decoration] {
+            if (decoration->resource()) {
+                wl_resource_destroy(decoration->resource());
+                delete decoration;
+            }
+        }
+    );
     decoration->setMode(defaultMode);
-    Q_EMIT q->decorationCreated(decoration);
+    emit q->decorationCreated(decoration);
 }
 
 ServerSideDecorationManagerInterface::Mode ServerSideDecorationManagerInterface::defaultMode() const
@@ -85,8 +109,7 @@ ServerSideDecorationManagerInterface::Private::Private(ServerSideDecorationManag
 {
 }
 
-namespace
-{
+namespace {
 static uint32_t modeWayland(ServerSideDecorationManagerInterface::Mode mode)
 {
     switch (mode) {
@@ -135,7 +158,7 @@ ServerSideDecorationManagerInterface::~ServerSideDecorationManagerInterface() =
 
 ServerSideDecorationManagerInterface::Private *ServerSideDecorationManagerInterface::d_func() const
 {
-    return reinterpret_cast<ServerSideDecorationManagerInterface::Private *>(d.data());
+    return reinterpret_cast<ServerSideDecorationManagerInterface::Private*>(d.data());
 }
 
 void ServerSideDecorationManagerInterface::setDefaultMode(Mode mode)
@@ -151,8 +174,8 @@ void ServerSideDecorationManagerInterface::setDefaultMode(Mode mode)
 class ServerSideDecorationInterface::Private : public Resource::Private
 {
 public:
-    Private(ServerSideDecorationInterface *q, ServerSideDecorationManagerInterface *c, SurfaceInterface *surface, wl_resource *parentResource);
-    ~Private() override;
+    Private(ServerSideDecorationInterface *q, ServerSideDecorationManagerInterface *c, SurfaceInterface *surface,  wl_resource *parentResource);
+    ~Private();
 
     ServerSideDecorationManagerInterface::Mode mode = ServerSideDecorationManagerInterface::Mode::None;
     SurfaceInterface *surface;
@@ -160,22 +183,33 @@ public:
     static ServerSideDecorationInterface *get(SurfaceInterface *s);
 
 private:
+    static void releaseCallback(wl_client *client, wl_resource *resource);
     static void requestModeCallback(wl_client *client, wl_resource *resource, uint32_t mode);
 
-    ServerSideDecorationInterface *q_func()
-    {
+    ServerSideDecorationInterface *q_func() {
         return reinterpret_cast<ServerSideDecorationInterface *>(q);
     }
 
     static const struct org_kde_kwin_server_decoration_interface s_interface;
-    static QVector<Private *> s_all;
+    static QVector<Private*> s_all;
 };
 
-#ifndef K_DOXYGEN
-const struct org_kde_kwin_server_decoration_interface ServerSideDecorationInterface::Private::s_interface = {resourceDestroyedCallback, requestModeCallback};
-QVector<ServerSideDecorationInterface::Private *> ServerSideDecorationInterface::Private::s_all;
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+const struct org_kde_kwin_server_decoration_interface ServerSideDecorationInterface::Private::s_interface = {
+    releaseCallback,
+    requestModeCallback
+};
+QVector<ServerSideDecorationInterface::Private*> ServerSideDecorationInterface::Private::s_all;
 #endif
 
+void ServerSideDecorationInterface::Private::releaseCallback(wl_client *client, wl_resource *resource)
+{
+    Q_UNUSED(client)
+    Private *p = cast<Private>(resource);
+    wl_resource_destroy(resource);
+    p->q->deleteLater();
+}
+
 void ServerSideDecorationInterface::Private::requestModeCallback(wl_client *client, wl_resource *resource, uint32_t mode)
 {
     Q_UNUSED(client)
@@ -195,24 +229,19 @@ void ServerSideDecorationInterface::Private::requestModeCallback(wl_client *clie
         qCWarning(KWAYLAND_SERVER) << "Invalid mode:" << mode;
         return;
     }
-    Q_EMIT cast<Private>(resource)->q_func()->modeRequested(m);
+    emit cast<Private>(resource)->q_func()->modeRequested(m);
 }
 
 ServerSideDecorationInterface *ServerSideDecorationInterface::Private::get(SurfaceInterface *s)
 {
-    auto it = std::find_if(s_all.constBegin(), s_all.constEnd(), [s](Private *p) {
-        return p->surface == s;
-    });
+    auto it = std::find_if(s_all.constBegin(), s_all.constEnd(), [s] (Private *p) { return p->surface == s; });
     if (it == s_all.constEnd()) {
         return nullptr;
     }
     return (*it)->q_func();
 }
 
-ServerSideDecorationInterface::Private::Private(ServerSideDecorationInterface *q,
-                                                ServerSideDecorationManagerInterface *c,
-                                                SurfaceInterface *surface,
-                                                wl_resource *parentResource)
+ServerSideDecorationInterface::Private::Private(ServerSideDecorationInterface *q, ServerSideDecorationManagerInterface *c, SurfaceInterface *surface, wl_resource *parentResource)
     : Resource::Private(q, c, parentResource, &org_kde_kwin_server_decoration_interface, &s_interface)
     , surface(surface)
 {
@@ -221,12 +250,14 @@ ServerSideDecorationInterface::Private::Private(ServerSideDecorationInterface *q
 
 ServerSideDecorationInterface::Private::~Private()
 {
+    if (resource) {
+        wl_resource_destroy(resource);
+        resource = nullptr;
+    }
     s_all.removeAll(this);
 }
 
-ServerSideDecorationInterface::ServerSideDecorationInterface(ServerSideDecorationManagerInterface *parent,
-                                                             SurfaceInterface *surface,
-                                                             wl_resource *parentResource)
+ServerSideDecorationInterface::ServerSideDecorationInterface(ServerSideDecorationManagerInterface *parent, SurfaceInterface *surface, wl_resource *parentResource)
     : Resource(new Private(this, parent, surface, parentResource))
 {
 }
@@ -255,7 +286,7 @@ SurfaceInterface *ServerSideDecorationInterface::surface() const
 
 ServerSideDecorationInterface::Private *ServerSideDecorationInterface::d_func() const
 {
-    return reinterpret_cast<ServerSideDecorationInterface::Private *>(d.data());
+    return reinterpret_cast<ServerSideDecorationInterface::Private*>(d.data());
 }
 
 ServerSideDecorationInterface *ServerSideDecorationInterface::get(SurfaceInterface *s)
@@ -265,5 +296,3 @@ ServerSideDecorationInterface *ServerSideDecorationInterface::get(SurfaceInterfa
 
 }
 }
-
-#include "moc_server_decoration_interface.cpp"
